var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "commands/commandBase", "common/pagedResultSet", "models/facet", "models/document"], function(require, exports, commandBase, pagedResultSet, facet, document) {
    var queryFacetsCommand = (function (_super) {
        __extends(queryFacetsCommand, _super);
        function queryFacetsCommand(indexName, queryText, skip, take, facets, db) {
            _super.call(this);
            this.indexName = indexName;
            this.queryText = queryText;
            this.skip = skip;
            this.take = take;
            this.facets = facets;
            this.db = db;
        }
        queryFacetsCommand.prototype.execute = function () {
            var _this = this;
            var argsUrl = this.urlEncodeArgs({
                query: this.queryText ? this.queryText : undefined,
                facetStart: this.skip,
                facetPageSize: this.take,
                facets: JSON.stringify(this.facets)
            });
            var url = "/facets/" + this.indexName + argsUrl;

            var finishedTask = $.Deferred();
            this.query(url, null, this.db).fail(function (response) {
                _this.reportError("Unable to run query.", response.responseText, response.statusText);
                finishedTask.reject(response);
            }).done(function (results) {
                finishedTask.resolve(_this.parseResults(results));
            });

            return finishedTask;
        };

        queryFacetsCommand.prototype.parseResults = function (resultSet) {
            var items = [];
            var totalItemCount = 0;

            var propIndex = 0;
            for (var prop in resultSet.Results) {
                var facetResult = resultSet.Results[prop];
                var propNameParts = prop.split('-');
                var facetName = propNameParts[0];
                var aggregateField = propNameParts[1];
                var correspondingFacet = this.facets[propIndex];
                var facetAggregationLabel = facet.getLabelForAggregation(correspondingFacet.Aggregation);
                totalItemCount = facetResult.Values.length + facetResult.RemainingTermsCount + this.skip;

                for (var i = 0; i < facetResult.Values.length; i++) {
                    var facetValue = facetResult.Values[i];
                    var item = items[i];
                    if (!item) {
                        item = new document({});
                        items[i] = item;
                    }

                    item['Key'] = facetValue.Range;

                    for (var power = 0; power < 5; power++) {
                        var curFieldName = facet.getLabelForAggregation(Math.pow(2, power));
                        var curFieldVal = facetValue[curFieldName];

                        if (!!curFieldVal) {
                            if (typeof curFieldVal === "number") {
                                var fixedVal = curFieldVal.toFixed(2);
                                if (fixedVal != curFieldVal) {
                                    curFieldVal = fixedVal;
                                }
                            }
                            item[curFieldName + " of " + aggregateField] = curFieldVal;
                        }
                    }
                }

                propIndex++;
            }

            return new pagedResultSet(items, totalItemCount, resultSet.Duration);
        };
        return queryFacetsCommand;
    })(commandBase);

    
    return queryFacetsCommand;
});
