define(["require", "exports", "durandal/composition", "ace/ace", "ace/ext/language_tools", "ace/mode/lucene", "ace/theme/github"], function(require, exports, composition, ace) {
    var aceEditorBindingHandler = (function () {
        function aceEditorBindingHandler() {
            this.defaults = {
                theme: "ace/theme/github",
                fontSize: "16px",
                lang: "ace/mode/csharp",
                readOnly: false
            };
        }
        aceEditorBindingHandler.install = function () {
            if (!ko.bindingHandlers["aceEditor"]) {
                ko.bindingHandlers["aceEditor"] = new aceEditorBindingHandler();

                composition.addBindingHandler("aceEditor");

                var Editor = require("ace/editor").Editor;
                require("ace/config").defineOptions(Editor.prototype, "editor", {
                    editorType: {
                        set: function (val) {
                        },
                        value: "general"
                    }
                });
            }
        };

        aceEditorBindingHandler.autoCompleteHub = function (editor, session, pos, prefix, callback) {
            var curEditorType = editor.getOption("editorType");
            var completerThreesome = aceEditorBindingHandler.customCompleters.first(function (x) {
                return x.editorType === curEditorType;
            });

            if (!!completerThreesome) {
                completerThreesome.completer.call(completerThreesome.containingViewModel, editor, session, pos, prefix, callback);
            }
        };

        aceEditorBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var bindingValues = valueAccessor();
            var theme = bindingValues.theme || this.defaults.theme;
            var fontSize = bindingValues.fontSize || this.defaults.fontSize;
            var lang = bindingValues.lang || this.defaults.lang;
            var readOnly = bindingValues.readOnly || this.defaults.readOnly;
            var typeName = bindingValues.typeName;
            var code = typeof bindingValues.code === "function" ? bindingValues.code : bindingContext.$rawData;
            var langTools = null;
            var containingViewModel = bindingValues.containigViewModel;

            if (typeof code !== "function") {
                throw new Error("code should be an observable");
            }

            if (!!bindingValues.completer) {
                langTools = ace.require("ace/ext/language_tools");
            }

            var aceEditor = ace.edit(element);

            aceEditor.setOption("enableBasicAutocompletion", true);
            aceEditor.setTheme(theme);
            aceEditor.setFontSize(fontSize);
            aceEditor.getSession().setMode(lang);
            aceEditor.setReadOnly(readOnly);

            if (!!typeName) {
                aceEditor.setOption("editorType", typeName);

                if (!!langTools) {
                    if (!aceEditorBindingHandler.customCompleters.first(function (x) {
                        return x.editorType === typeName;
                    })) {
                        aceEditorBindingHandler.customCompleters.push({ editorType: typeName, containingViewModel: containingViewModel, completer: bindingValues.completer });
                    }
                    if (!!aceEditor.completers) {
                        var completersList = aceEditor.completers;
                        if (!completersList.first(function (x) {
                            return x.moduleId === "aceEditoBindingHandler";
                        })) {
                            langTools.addCompleter({ moduleId: "aceEditoBindingHandler", getCompletions: aceEditorBindingHandler.autoCompleteHub });
                        }
                    } else {
                        langTools.addCompleter({ moduleId: "aceEditoBindingHandler", getCompletions: aceEditorBindingHandler.autoCompleteHub });
                    }
                }
            }

            var aceFocusElement = ".ace_text-input";

            $(element).on('keyup', aceFocusElement, function () {
                return code(aceEditor.getSession().getValue());
            });
            $(element).on('focus', aceFocusElement, function () {
                return aceEditorBindingHandler.currentEditor = aceEditor;
            });

            ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                return $(element).off('blur', aceFocusElement);
            });

            ko.utils.domData.set(element, "aceEditor", aceEditor);

            if (bindingValues.getFocus && bindingValues.getFocus == true) {
                aceEditor.focus();
            }
        };

        aceEditorBindingHandler.prototype.update = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var bindingValues = valueAccessor();
            var code = ko.unwrap(bindingValues.code);
            var aceEditor = ko.utils.domData.get(element, "aceEditor");
            var editorCode = aceEditor.getSession().getValue();
            if (code !== editorCode) {
                aceEditor.getSession().setValue(code);
            }
        };
        aceEditorBindingHandler.customCompleters = [];
        return aceEditorBindingHandler;
    })();

    
    return aceEditorBindingHandler;
});
