define(["require", "exports", "models/database", "models/filesystem/filesystem", "plugins/router"], function(require, exports, database, filesystem, router) {
    var appUrl = (function () {
        function appUrl() {
        }
        appUrl.checkIsAreaActive = function (routeRoot) {
            var items = router.routes.filter(function (m) {
                return m.isActive() && m.route != null && m.route != '';
            });
            var isThereAny = items.some(function (m) {
                return m.route.substring(0, routeRoot.length) === routeRoot;
            });

            return isThereAny;
        };

        appUrl.forDatabases = function () {
            return "#databases";
        };

        appUrl.forFilesystems = function () {
            return "#filesystems";
        };

        appUrl.forEditDoc = function (id, collectionName, docIndexInCollection, db) {
            var databaseUrlPart = appUrl.getEncodedDbPart(db);
            var docIdUrlPart = id ? "&id=" + encodeURIComponent(id) : "";
            var pagedListInfo = collectionName && docIndexInCollection != null ? "&list=" + encodeURIComponent(collectionName) + "&item=" + docIndexInCollection : "";
            return "#databases/edit?" + docIdUrlPart + databaseUrlPart + pagedListInfo;
        };

        appUrl.forEditItem = function (itemId, res, itemIndex, collectionName) {
            var databaseUrlPart = appUrl.getEncodedResourcePart(res);
            var itemIdUrlPart = itemId ? "&id=" + encodeURIComponent(itemId) : "";

            var pagedListInfo = collectionName && itemIndex != null ? "&list=" + encodeURIComponent(collectionName) + "&item=" + itemIndex : "";
            var resourceTag = res instanceof filesystem ? "#filesystems" : "#databases";
            return resourceTag + "/edit?" + itemIdUrlPart + databaseUrlPart + pagedListInfo;
        };

        appUrl.forNewDoc = function (db) {
            var databaseUrlPart = appUrl.getEncodedDbPart(db);
            return "#databases/edit?" + databaseUrlPart;
        };

        appUrl.forStatus = function (db) {
            return "#databases/status?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forSettings = function (db) {
            var path = (db && db.isSystem) ? "#databases/settings/apiKeys" : "#databases/settings/databaseSettings?" + appUrl.getEncodedDbPart(db);
            return path;
        };

        appUrl.forLogs = function (db) {
            return "#databases/status/logs?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forAlerts = function (db) {
            return "#databases/status/alerts?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forIndexErrors = function (db) {
            return "#databases/status/indexErrors?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forReplicationStats = function (db) {
            return "#databases/status/replicationStats?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forUserInfo = function (db) {
            return "#databases/status/userInfo?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forApiKeys = function () {
            return "#databases/settings/apiKeys";
        };

        appUrl.forWindowsAuth = function () {
            return "#databases/settings/windowsAuth";
        };

        appUrl.forDatabaseSettings = function (db) {
            return "#databases/settings/databaseSettings?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forPeriodicBackup = function (db) {
            return "#databases/settings/periodicBackups?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forReplications = function (db) {
            return "#databases/settings/replication?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forSqlReplications = function (db) {
            return "#databases/settings/sqlReplication?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forScriptedIndexes = function (db) {
            return "#databases/settings/scriptedIndex?" + appUrl.getEncodedDbPart(db);
        };

        appUrl.forDocuments = function (collection, db) {
            var collectionPart = collection ? "collection=" + encodeURIComponent(collection) : "";
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/documents?" + collectionPart + databasePart;
        };

        appUrl.forConflicts = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/conflicts?" + databasePart;
        };

        appUrl.forPatch = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/patch?" + databasePart;
        };

        appUrl.forIndexes = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes?" + databasePart;
        };

        appUrl.forNewIndex = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/edit?" + databasePart;
        };

        appUrl.forEditIndex = function (indexName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/edit/" + encodeURIComponent(indexName) + "?" + databasePart;
        };

        appUrl.forNewTransformer = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers/edit?" + databasePart;
        };

        appUrl.forEditTransformer = function (transformerName, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers/edit/" + encodeURIComponent(transformerName) + "?" + databasePart;
        };

        appUrl.forTransformers = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/transformers?" + databasePart;
        };

        appUrl.forQuery = function (db, indexNameOrHashToQuery) {
            var databasePart = appUrl.getEncodedDbPart(db);
            var indexToQueryComponent = indexNameOrHashToQuery;
            if (typeof indexNameOrHashToQuery === "number") {
                indexToQueryComponent = "recentquery-" + indexNameOrHashToQuery;
            }

            var indexPart = indexToQueryComponent ? "/" + encodeURIComponent(indexToQueryComponent) : "";
            return "#databases/query/index" + indexPart + "?" + databasePart;
        };

        appUrl.forReporting = function (db, indexName) {
            var databasePart = appUrl.getEncodedDbPart(db);
            var indexPart = indexName ? "/" + encodeURIComponent(indexName) : "";
            return "#databases/query/reporting" + indexPart + "?" + databasePart;
        };

        appUrl.forTasks = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks?" + databasePart;
        };

        appUrl.forResourceQuery = function (res) {
            if (res && res instanceof database && !res.isSystem) {
                return appUrl.baseUrl + "/databases/" + res.name;
            } else if (res && res instanceof filesystem) {
                return appUrl.baseUrl + "/ravenfs/" + res.name;
            }

            return this.baseUrl;
        };

        appUrl.forTerms = function (index, db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/indexes/terms/" + encodeURIComponent(index) + "?" + databasePart;
        };

        appUrl.forImportDatabase = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/importDatabase?" + databasePart;
        };

        appUrl.forExportDatabase = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/exportDatabase?" + databasePart;
        };

        appUrl.forBackupDatabase = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/backupDatabase?" + databasePart;
        };

        appUrl.forRestoreDatabase = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/restoreDatabase?" + databasePart;
        };

        appUrl.forToggleIndexing = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/toggleIndexing?" + databasePart;
        };

        appUrl.forSampleData = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/sampleData?" + databasePart;
        };

        appUrl.forCsvImport = function (db) {
            var databasePart = appUrl.getEncodedDbPart(db);
            return "#databases/tasks/csvImport?" + databasePart;
        };

        appUrl.forFilesystem = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems?" + filesystemPart;
        };

        appUrl.forIndexesRawData = function (db) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/indexes";
        };

        appUrl.forIndexQueryRawData = function (db, indexName) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/indexes/" + indexName;
        };

        appUrl.forTransformersRawData = function (db) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/transformers";
        };

        appUrl.forDatabasesRawData = function () {
            return window.location.protocol + "//" + window.location.host + "/databases";
        };

        appUrl.forDocumentRawData = function (db, docId) {
            return window.location.protocol + "//" + window.location.host + "/databases/" + db.name + "/docs/" + docId;
        };

        appUrl.forFilesystemFiles = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/files?" + filesystemPart;
        };

        appUrl.forFilesystemSearch = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/search?" + filesystemPart;
        };

        appUrl.forFilesystemSynchronization = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/synchronization?" + filesystemPart;
        };

        appUrl.forFilesystemConfiguration = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/configuration?" + filesystemPart;
        };

        appUrl.forFilesystemConfigurationWithKey = function (fs, key) {
            var filesystemPart = appUrl.getEncodedFsPart(fs) + "&key=" + encodeURIComponent(key);
            return "#filesystems/configuration?" + filesystemPart;
        };

        appUrl.forFilesystemUploadFile = function (fs) {
            var filesystemPart = appUrl.getEncodedFsPart(fs);
            return "#filesystems/upload?" + filesystemPart;
        };

        appUrl.getResource = function () {
            var appFilesystem = appUrl.getFilesystem();
            if (!appFilesystem.isDefault) {
                return appFilesystem;
            } else {
                return appUrl.getDatabase();
            }
        };

        appUrl.getDatabase = function () {
            var dbIndicator = "database=";
            var hash = window.location.hash;
            var dbIndex = hash.indexOf(dbIndicator);
            if (dbIndex >= 0) {
                var dbSegmentEnd = hash.indexOf("&", dbIndex);
                if (dbSegmentEnd === -1) {
                    dbSegmentEnd = hash.length;
                }

                var databaseName = hash.substring(dbIndex + dbIndicator.length, dbSegmentEnd);
                var unescapedDatabaseName = decodeURIComponent(databaseName);
                var db = new database(unescapedDatabaseName);
                db.isSystem = unescapedDatabaseName === "<system>";
                return db;
            } else {
                return null;
            }
        };

        appUrl.getSystemDatabase = function () {
            var db = new database("<system>");
            db.isSystem = true;
            return db;
        };

        appUrl.getFilesystem = function () {
            var filesystemIndicator = "filesystem=";
            var hash = window.location.hash;
            var fsIndex = hash.indexOf(filesystemIndicator);
            if (fsIndex >= 0) {
                var fsSegmentEnd = hash.indexOf("&", fsIndex);
                if (fsSegmentEnd === -1) {
                    fsSegmentEnd = hash.length;
                }

                var filesystemName = hash.substring(fsIndex + filesystemIndicator.length, fsSegmentEnd);
                var unescapedDatabaseName = decodeURIComponent(filesystemName);
                var fs = new filesystem(unescapedDatabaseName);
                fs.isDefault = unescapedDatabaseName === "<default>";
                return fs;
            } else {
                return this.getDefaultFilesystem();
            }
        };

        appUrl.getDefaultFilesystem = function () {
            var fs = new filesystem("<default>");
            fs.isDefault = true;
            return fs;
        };

        appUrl.forServer = function () {
            if (window.location.protocol === "file:") {
                if (window.location.search.indexOf("fiddler")) {
                    return "http://localhost.fiddler:8080";
                } else {
                    return "http://localhost:8080";
                }
            }

            return window.location.protocol + "//" + window.location.host;
        };

        appUrl.forCurrentPage = function (rs) {
            var routerInstruction = router.activeInstruction();
            if (routerInstruction) {
                var dbNameInAddress = routerInstruction.queryParams ? routerInstruction.queryParams[rs.type] : null;
                var isDifferentDbInAddress = !dbNameInAddress || dbNameInAddress !== rs.name.toLowerCase();
                if (isDifferentDbInAddress) {
                    var existingAddress = window.location.hash;
                    var existingDbQueryString = dbNameInAddress ? rs.type + "=" + encodeURIComponent(dbNameInAddress) : null;
                    var newDbQueryString = rs.type + "=" + encodeURIComponent(rs.name);
                    var newUrlWithDatabase = existingDbQueryString ? existingAddress.replace(existingDbQueryString, newDbQueryString) : existingAddress + (window.location.hash.indexOf("?") >= 0 ? "&" : "?") + rs.type + "=" + encodeURIComponent(rs.name);

                    return newUrlWithDatabase;
                }
            }
        };

        appUrl.forCurrentDatabase = function () {
            return appUrl.currentDbComputeds;
        };

        appUrl.getEncodedResourcePart = function (res) {
            if (!res)
                return "";
            if (res instanceof filesystem) {
                return appUrl.getEncodedFsPart(res);
            } else {
                return appUrl.getEncodedDbPart(res);
            }
        };

        appUrl.getEncodedDbPart = function (db) {
            return db ? "&database=" + encodeURIComponent(db.name) : "";
        };

        appUrl.getEncodedFsPart = function (fs) {
            return fs ? "&filesystem=" + encodeURIComponent(fs.name) : "";
        };
        appUrl.baseUrl = "";
        appUrl.currentDatabase = ko.observable().subscribeTo("ActivateDatabase", true);
        appUrl.currentFilesystem = ko.observable().subscribeTo("ActivateFilesystem", true);

        appUrl.currentDbComputeds = {
            databases: ko.computed(function () {
                return appUrl.forDatabases();
            }),
            documents: ko.computed(function () {
                return appUrl.forDocuments(null, appUrl.currentDatabase());
            }),
            conflicts: ko.computed(function () {
                return appUrl.forConflicts(appUrl.currentDatabase());
            }),
            patch: ko.computed(function () {
                return appUrl.forPatch(appUrl.currentDatabase());
            }),
            indexes: ko.computed(function () {
                return appUrl.forIndexes(appUrl.currentDatabase());
            }),
            transformers: ko.computed(function () {
                return appUrl.forTransformers(appUrl.currentDatabase());
            }),
            newIndex: ko.computed(function () {
                return appUrl.forNewIndex(appUrl.currentDatabase());
            }),
            editIndex: function (indexName) {
                return ko.computed(function () {
                    return appUrl.forEditIndex(indexName, appUrl.currentDatabase());
                });
            },
            newTransformer: ko.computed(function () {
                return appUrl.forNewTransformer(appUrl.currentDatabase());
            }),
            editTransformer: function (transformerName) {
                return ko.computed(function () {
                    return appUrl.forEditTransformer(transformerName, appUrl.currentDatabase());
                });
            },
            query: function (indexName) {
                return ko.computed(function () {
                    return appUrl.forQuery(appUrl.currentDatabase(), indexName);
                });
            },
            reporting: ko.computed(function () {
                return appUrl.forReporting(appUrl.currentDatabase());
            }),
            tasks: ko.computed(function () {
                return appUrl.forTasks(appUrl.currentDatabase());
            }),
            status: ko.computed(function () {
                return appUrl.forStatus(appUrl.currentDatabase());
            }),
            settings: ko.computed(function () {
                return appUrl.forSettings(appUrl.currentDatabase());
            }),
            logs: ko.computed(function () {
                return appUrl.forLogs(appUrl.currentDatabase());
            }),
            alerts: ko.computed(function () {
                return appUrl.forAlerts(appUrl.currentDatabase());
            }),
            indexErrors: ko.computed(function () {
                return appUrl.forIndexErrors(appUrl.currentDatabase());
            }),
            replicationStats: ko.computed(function () {
                return appUrl.forReplicationStats(appUrl.currentDatabase());
            }),
            userInfo: ko.computed(function () {
                return appUrl.forUserInfo(appUrl.currentDatabase());
            }),
            databaseSettings: ko.computed(function () {
                return appUrl.forDatabaseSettings(appUrl.currentDatabase());
            }),
            periodicBackup: ko.computed(function () {
                return appUrl.forPeriodicBackup(appUrl.currentDatabase());
            }),
            replications: ko.computed(function () {
                return appUrl.forReplications(appUrl.currentDatabase());
            }),
            sqlReplications: ko.computed(function () {
                return appUrl.forSqlReplications(appUrl.currentDatabase());
            }),
            scriptedIndexes: ko.computed(function () {
                return appUrl.forScriptedIndexes(appUrl.currentDatabase());
            }),
            isAreaActive: function (routeRoot) {
                return ko.computed(function () {
                    return appUrl.checkIsAreaActive(routeRoot);
                });
            },
            isActive: function (routeTitle) {
                return ko.computed(function () {
                    return router.navigationModel().first(function (m) {
                        return m.isActive() && m.title === routeTitle;
                    }) != null;
                });
            },
            databasesManagement: ko.computed(function () {
                return appUrl.forDatabases() + "?database=" + appUrl.getEncodedDbPart(appUrl.currentDatabase());
            }),
            filesystems: ko.computed(function () {
                return appUrl.forFilesystems();
            }),
            filesystemsManagement: ko.computed(function () {
                return appUrl.forFilesystems() + "?filesystem=" + appUrl.getEncodedFsPart(appUrl.currentFilesystem());
            }),
            filesystemFiles: ko.computed(function () {
                return appUrl.forFilesystemFiles(appUrl.currentFilesystem());
            }),
            filesystemSearch: ko.computed(function () {
                return appUrl.forFilesystemSearch(appUrl.currentFilesystem());
            }),
            filesystemSynchronization: ko.computed(function () {
                return appUrl.forFilesystemSynchronization(appUrl.currentFilesystem());
            }),
            filesystemConfiguration: ko.computed(function () {
                return appUrl.forFilesystemConfiguration(appUrl.currentFilesystem());
            })
        };

        appUrl.warnWhenUsingSystemDatabase = true;
        return appUrl;
    })();

    
    return appUrl;
});
