﻿define(["require", "exports", "durandal/composition"], function(require, exports, composition) {
    var autoCompleteBindingHandler = (function () {
        function autoCompleteBindingHandler() {
        }
        autoCompleteBindingHandler.install = function () {
            if (!ko.bindingHandlers["autoComplete"]) {
                ko.bindingHandlers["autoComplete"] = new autoCompleteBindingHandler();

                composition.addBindingHandler("autoComplete");
            }
        };

        autoCompleteBindingHandler.prototype.init = function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            var inputId = valueAccessor();
            var input = $(inputId);
            if (input.length !== 1) {
                throw new Error("Expected 1 auto complete item, '" + inputId + "', but found " + input.length);
            }

            var $element = $(element);
            element.style.display = "none";
            element.style.position = "absolute";
            element.style.left = "auto";
            element.style.width = input.width() + "px";
            element.style.top = (input.height() + 20) + "px";

            $element.on('click', function () {
                return setTimeout(function () {
                    return element.style.display = "none";
                }, 0);
            });

            input.on('blur', function (args) {
                return setTimeout(function () {
                    return element.style.display = "none";
                }, 200);
            });

            input.on('keydown', function (args) {
                var lis, curSelected;
                if (args.which == 40 || args.which == 38 || args.which == 13) {
                    lis = $element.children("li");
                    curSelected = $element.find('.selected');
                }

                if (args.which == 40) {
                    if (curSelected.length > 0) {
                        curSelected.removeClass("selected");
                        var nextSelected = curSelected.next();

                        if (nextSelected.length) {
                            nextSelected.addClass("selected");
                        } else {
                            lis.first().addClass('selected');
                        }
                    } else {
                        curSelected = lis.first().addClass("selected");
                    }
                } else if (args.which == 38) {
                    if (curSelected.length > 0) {
                        curSelected.removeClass("selected");
                        var prevSelected = curSelected.prev();

                        if (prevSelected.length) {
                            prevSelected.addClass("selected");
                        } else {
                            lis.last().addClass('selected');
                        }
                    } else {
                        curSelected = lis.last().addClass("selected");
                    }
                } else if (args.which == 13) {
                    if (curSelected.length) {
                        curSelected.click();
                    }
                }
            });

            var results = allBindings()['foreach'];
            if (!results) {
                throw new Error("Unable to find results list for auto complete.");
            }
            var subscription = results.subscribe(function (array) {
                element.style.display = array.length === 0 ? "none" : "block";
            });

            ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                input.off('blur');
                $element.off('click');
                input.off('keydown');
                subscription.dispose();
            });
        };
        return autoCompleteBindingHandler;
    })();

    
    return autoCompleteBindingHandler;
});
