var subscribableFn = ko.subscribable.fn;
var observableArrayFn = ko.observableArray.fn;

subscribableFn.where = function (predicate) {
    var observable = this;
    var matches = ko.observable();
    observable.subscribe(function (val) {
        if (predicate(val)) {
            matches(val);
        }
    });
    return matches;
};

subscribableFn.distinctUntilChanged = function () {
    var observable = this;
    var matches = ko.observable();
    var lastMatch = observable();
    observable.subscribe(function (val) {
        if (val !== lastMatch) {
            lastMatch = val;
            matches(val);
        }
    });
    return matches;
};

subscribableFn.throttle = function (throttleTimeMs) {
    var observable = this;
    return ko.computed(function () {
        return observable();
    }).extend({ throttle: throttleTimeMs });
};

subscribableFn.select = function (selector) {
    var observable = this;
    var selectedResults = ko.observable();
    observable.subscribe(function (val) {
        return selectedResults(selector(val));
    });
    return selectedResults;
};

subscribableFn.toggle = function () {
    var observable = this;
    observable(!observable());
    return observable;
};

observableArrayFn.pushAll = function (items) {
    this.push.apply(this, items);
};

observableArrayFn.contains = function (item) {
    return this.indexOf(item) !== -1;
};

observableArrayFn.first = function (filter) {
    return this().first(filter);
};

observableArrayFn.last = function (filter) {
    return this().last(filter);
};

var functionPrototype = Function.prototype;
functionPrototype.memoize = function (thisVal) {
    var self = this;
    var cache = {};
    return function (arg) {
        if (arg in cache) {
            return cache[arg];
        } else {
            return cache[arg] = self.call(thisVal, arg);
        }
    };
};


var arrayPrototype = Array.prototype;
arrayPrototype.remove = function (item) {
    var self = this;
    var index = self.indexOf(item);
    if (index >= 0) {
        self.splice(index, 1);
    }
    return index;
};

arrayPrototype.removeAll = function (items) {
    var i = 0;
    var self = this;
    for (var i = self.length - 1; i >= 0 && items.length > 0; i--) {
        var itemsIndex = items.indexOf(self[i]);
        if (itemsIndex >= 0) {
            self.splice(i, 1);
            items.splice(itemsIndex);
        }
    }
};

arrayPrototype.first = function (filter) {
    var self = this;
    if (self.length > 0) {
        if (filter) {
            return ko.utils.arrayFirst(self, filter);
        } else if (self.length > 0) {
            return self[0];
        }
    }

    return null;
};

arrayPrototype.last = function (filter) {
    var self = this;
    if (filter) {
        for (var i = self.length - 1; i > 0; i--) {
            if (filter(self[i])) {
                return self[i];
            }
        }
    } else if (self.length > 0) {
        return self[self.length - 1];
    }

    return null;
};

arrayPrototype.pushAll = function (items) {
    this.push.apply(this, items);
};

arrayPrototype.contains = function (item) {
    var self = this;
    return self.indexOf(item) !== -1;
};

arrayPrototype.count = function (filter) {
    var self = this;
    if (filter) {
        var matches = 0;
        for (var i = 0; i < self.length; i++) {
            if (filter(self[i])) {
                matches++;
            }
        }

        return matches;
    }

    return self.length;
};

arrayPrototype.distinct = function () {
    var distinctElements = [];
    for (var i = 0; i < this.length; i++) {
        var element = this[i];
        if (!distinctElements.contains(element)) {
            distinctElements.push(element);
        }
    }

    return distinctElements;
};


String.prototype.hashCode = function () {
    var hash = 0;
    if (this.length == 0)
        return hash;
    for (var i = 0; i < this.length; i++) {
        var char = this.charCodeAt(i);
        hash = ((hash << 5) - hash) + char;
        hash |= 0;
    }
    return hash;
};

String.prototype.replaceAll = function (find, replace) {
    return this.replace(new RegExp(find, 'g'), replace);
};

String.prototype.reverse = function (input) {
    var chars = new Array;
    if (input != null) {
        chars = input.split('');
    }
    return chars.reverse().join('');
};

ko.bindingHandlers['slideVisible'] = {
    init: function (element, valueAccessor) {
        var value = valueAccessor();
        jQuery(element).toggle(ko.unwrap(value));
    },
    update: function (element, valueAccessor) {
        var value = valueAccessor();
        ko.unwrap(value) ? jQuery(element).slideDown() : jQuery(element).slideUp();
    }
};

var Pair = (function () {
    function Pair(item1, item2) {
        this.item1 = item1;
        this.item2 = item2;
    }
    return Pair;
})();
