define(["require", "exports"], function(require, exports) {
    var facet = (function () {
        function facet(dto) {
            var _this = this;
            this.aggregation = ko.observable();
            this.aggregationLabel = ko.computed(function () {
                return facet.getLabelForAggregation(_this.aggregation());
            });
            this.aggregation(dto.Aggregation);
            this.aggregationField = dto.AggregationField;
            this.aggregationType = dto.AggregationType;
            this.displayName = dto.DisplayName;
            this.includeRemainingTerms = dto.IncludeRemainingTerms;
            this.maxResults = dto.MaxResults;
            this.mode = dto.Mode;
            this.name = dto.Name;
            this.ranges = dto.Ranges;
            this.termSortMode = dto.TermSortMode;
        }
        facet.prototype.toDto = function () {
            return {
                Aggregation: this.aggregation(),
                AggregationField: this.aggregationField,
                AggregationType: "System.Int32",
                DisplayName: this.displayName,
                IncludeRemainingTerms: this.includeRemainingTerms,
                MaxResults: this.maxResults,
                Mode: this.mode,
                Name: this.name,
                Ranges: this.ranges,
                TermSortMode: this.termSortMode
            };
        };

        facet.fromNameAndAggregation = function (name, aggregationField) {
            var dto = {
                Aggregation: 0,
                AggregationField: aggregationField,
                AggregationType: "System.Int32",
                DisplayName: name + "-" + aggregationField,
                IncludeRemainingTerms: false,
                MaxResults: null,
                Mode: 0,
                Name: name,
                Ranges: [],
                TermSortMode: 0
            };

            return new facet(dto);
        };

        facet.getLabelForAggregation = function (aggregation) {
            return aggregation === 1 ? "Count" : aggregation === 2 ? "Max" : aggregation === 4 ? "Min" : aggregation === 8 ? "Average" : aggregation === 16 ? "Sum" : "None";
        };

        facet.prototype.setAggregationToCount = function () {
            this.aggregation(1);
        };

        facet.prototype.setAggregationToMax = function () {
            this.aggregation(2);
        };

        facet.prototype.setAggregationToMin = function () {
            this.aggregation(4);
        };

        facet.prototype.setAggregationToAverage = function () {
            this.aggregation(8);
        };

        facet.prototype.setAggregationToSum = function () {
            this.aggregation(16);
        };
        return facet;
    })();

    
    return facet;
});
