var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "commands/getAlertsCommand", "viewmodels/viewModelBase", "moment", "models/alert", "commands/saveAlertsCommand"], function(require, exports, getAlertsCommand, viewModelBase, moment, alert, saveAlertsCommand) {
    var alerts = (function (_super) {
        __extends(alerts, _super);
        function alerts() {
            var _this = this;
            _super.call(this);
            this.alertDoc = ko.observable();
            this.allAlerts = ko.observableArray();
            this.filterLevel = ko.observable("All");
            this.selectedAlert = ko.observable();
            this.now = ko.observable();
            this.updateNowTimeoutHandle = 0;

            this.unreadAlertCount = ko.computed(function () {
                return _this.allAlerts().count(function (a) {
                    return a.observed() === false;
                });
            });
            this.readAlertCount = ko.computed(function () {
                return _this.allAlerts().count(function (a) {
                    return a.observed() === true;
                });
            });
            this.updateCurrentNowTime();
            this.activeDatabase.subscribe(function () {
                return _this.fetchAlerts();
            });
        }
        alerts.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);
            this.fetchAlerts();
        };

        alerts.prototype.deactivate = function () {
            clearTimeout(this.updateNowTimeoutHandle);
        };

        alerts.prototype.fetchAlerts = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                return new getAlertsCommand(db).execute().done(function (result) {
                    return _this.processAlertsResults(result);
                });
            }

            return null;
        };

        alerts.prototype.processAlertsResults = function (result) {
            var _this = this;
            var now = moment();
            var alerts = result.Alerts.map(function (a) {
                return new alert(a);
            });
            alerts.forEach(function (r) {
                r.createdAtHumanized = _this.createHumanReadableTime(r.createdAt), r.isVisible = ko.computed(function () {
                    return _this.matchesFilter(r);
                });
            });
            this.alertDoc(result);
            this.allAlerts(alerts);
        };

        alerts.prototype.matchesFilter = function (a) {
            if (this.filterLevel() === "All") {
                return true;
            }

            var unreadFilterWithUnreadAlert = this.filterLevel() === "Unread" && a.observed() === false;
            var readFilterWithReadAlert = this.filterLevel() === "Read" && a.observed() === true;
            return unreadFilterWithUnreadAlert || readFilterWithReadAlert;
        };

        alerts.prototype.createHumanReadableTime = function (time) {
            var _this = this;
            if (time) {
                return ko.computed(function () {
                    var dateMoment = moment(time);
                    var agoInMs = dateMoment.diff(_this.now());
                    return moment.duration(agoInMs).humanize(true) + dateMoment.format(" (MM/DD/YY, h:mma)");
                });
            }

            return ko.computed(function () {
                return time;
            });
        };

        alerts.prototype.selectAlert = function (selection) {
            this.selectedAlert(selection);
        };

        alerts.prototype.tableKeyDown = function (sender, e) {
            var isKeyUp = e.keyCode === 38;
            var isKeyDown = e.keyCode === 40;
            if (isKeyUp || isKeyDown) {
                e.preventDefault();

                var oldSelection = this.selectedAlert();
                if (oldSelection) {
                    var oldSelectionIndex = this.allAlerts.indexOf(oldSelection);
                    var newSelectionIndex = oldSelectionIndex;
                    if (isKeyUp && oldSelectionIndex > 0) {
                        newSelectionIndex--;
                    } else if (isKeyDown && oldSelectionIndex < this.allAlerts().length - 1) {
                        newSelectionIndex++;
                    }

                    this.selectedAlert(this.allAlerts()[newSelectionIndex]);
                    var newSelectedRow = $("#alertsContainer table tbody tr:nth-child(" + (newSelectionIndex + 1) + ")");
                    if (newSelectedRow) {
                        this.ensureRowVisible(newSelectedRow);
                    }
                }
            }
        };

        alerts.prototype.ensureRowVisible = function (row) {
            var table = $("#alertTableContainer");
            var scrollTop = table.scrollTop();
            var scrollBottom = scrollTop + table.height();
            var scrollHeight = scrollBottom - scrollTop;

            var rowPosition = row.position();
            var rowTop = rowPosition.top;
            var rowBottom = rowTop + row.height();

            if (rowTop < 0) {
                table.scrollTop(scrollTop + rowTop);
            } else if (rowBottom > scrollHeight) {
                table.scrollTop(scrollTop + (rowBottom - scrollHeight));
            }
        };

        alerts.prototype.setFilterAll = function () {
            this.filterLevel("All");
        };

        alerts.prototype.setFilterUnread = function () {
            this.filterLevel("Unread");
        };

        alerts.prototype.setFilterRead = function () {
            this.filterLevel("Read");
        };

        alerts.prototype.updateCurrentNowTime = function () {
            var _this = this;
            this.now(moment());
            this.updateNowTimeoutHandle = setTimeout(function () {
                return _this.updateCurrentNowTime();
            }, 60000);
        };

        alerts.prototype.toggleSelectedReadState = function () {
            var alert = this.selectedAlert();
            if (alert) {
                alert.observed(!alert.observed());
            }
        };

        alerts.prototype.deleteSelectedAlert = function () {
            var alert = this.selectedAlert();
            if (alert) {
                this.allAlerts.remove(alert);
            }
        };

        alerts.prototype.deleteReadAlerts = function () {
            this.allAlerts.remove(function (a) {
                return a.observed();
            });
        };

        alerts.prototype.deleteAllAlerts = function () {
            this.allAlerts.removeAll();
        };

        alerts.prototype.saveAlerts = function () {
            var alertDoc = this.alertDoc();
            var db = this.activeDatabase();
            if (alertDoc && db) {
                alertDoc.Alerts = this.allAlerts().map(function (a) {
                    return a.toDto();
                });
                new saveAlertsCommand(alertDoc, db).execute();
            }
        };
        return alerts;
    })(viewModelBase);

    
    return alerts;
});
