var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "durandal/app", "plugins/router", "common/appUrl", "models/database", "commands/getDatabaseStatsCommand", "commands/getDatabasesCommand", "viewmodels/viewModelBase", "commands/createDatabaseCommand", "viewmodels/createEncryption", "viewmodels/createEncryptionConfirmation", 'viewmodels/shell'], function(require, exports, app, router, appUrl, database, getDatabaseStatsCommand, getDatabasesCommand, viewModelBase, createDatabaseCommand, createEncryption, createEncryptionConfirmation, shell) {
    var databases = (function (_super) {
        __extends(databases, _super);
        function databases() {
            var _this = this;
            _super.call(this);
            this.databases = ko.observableArray();
            this.searchText = ko.observable("");
            this.selectedDatabase = ko.observable();

            this.systemDb = appUrl.getSystemDatabase();
            this.docsForSystemUrl = appUrl.forDocuments(null, this.systemDb);
            this.searchText.extend({ throttle: 200 }).subscribe(function (s) {
                return _this.filterDatabases(s);
            });
        }
        databases.prototype.canActivate = function (args) {
            return true;
        };

        databases.prototype.attached = function () {
            var _this = this;
            ko.postbox.publish("SetRawJSONUrl", appUrl.forDatabasesRawData());
            this.databasesChangeSubscription = shell.globalChangesApi.watchDocPrefix(function (e) {
                if (!!e.Id && e.Id.indexOf("Raven/Databases") == 0 && (e.Type == 1 /* Put */ || e.Type == 2 /* Delete */)) {
                    if (e.Type == 2 /* Delete */) {
                        _this.onDatabaseDeleted(new database(e.Id));
                    }

                    _this.modelPolling();
                }
            }, "Raven/Databases");
        };

        databases.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            this.databasesChangeSubscription.off();
        };

        databases.prototype.modelPolling = function () {
            var _this = this;
            new getDatabasesCommand().execute().done(function (results) {
                return _this.databasesLoaded(results);
            });
        };

        databases.prototype.navigateToDocuments = function (db) {
            db.activate();
            router.navigate(appUrl.forDocuments(null, db));
        };

        databases.prototype.getDocumentsUrl = function (db) {
            return appUrl.forDocuments(null, db);
        };

        databases.prototype.databasesLoaded = function (results) {
            var _this = this;
            var databasesHaveChanged = this.checkDifferentDatabases(results);
            if (databasesHaveChanged) {
                this.databases(results);

                var few = 20;
                if (results.length < few && !this.initializedStats) {
                    this.initializedStats = true;
                    results.forEach(function (db) {
                        return _this.fetchStats(db);
                    });
                }

                if (results.length === 0) {
                    this.newDatabase();
                }
            }
        };

        databases.prototype.checkDifferentDatabases = function (dbs) {
            if (dbs.length !== this.databases().length) {
                return true;
            }

            var freshDbNames = dbs.map(function (db) {
                return db.name;
            });
            var existingDbNames = this.databases().map(function (d) {
                return d.name;
            });
            return existingDbNames.some(function (existing) {
                return !freshDbNames.contains(existing);
            });
        };

        databases.prototype.newDatabase = function () {
            var _this = this;
            require(["viewmodels/createDatabase"], function (createDatabase) {
                var createDatabaseViewModel = new createDatabase(_this.databases);
                createDatabaseViewModel.creationTask.done(function (databaseName, bundles, databasePath, databaseLogs, databaseIndexes) {
                    var settings = {
                        "Raven/ActiveBundles": bundles.join(";")
                    };
                    settings["Raven/DataDir"] = (!_this.isEmptyStringOrWhitespace(databasePath)) ? databasePath : "~/Databases/" + databaseName;
                    if (!_this.isEmptyStringOrWhitespace(databaseLogs)) {
                        settings["Raven/Esent/LogsPath"] = databaseLogs;
                    }
                    if (!_this.isEmptyStringOrWhitespace(databaseIndexes)) {
                        settings["Raven/IndexStoragePath"] = databaseIndexes;
                    }

                    _this.showDbCreationAdvancedStepsIfNecessary(databaseName, bundles, settings);
                });
                app.showDialog(createDatabaseViewModel);
            });
        };

        databases.prototype.showDbCreationAdvancedStepsIfNecessary = function (databaseName, bundles, settings) {
            var _this = this;
            var securedSettings = {};
            var deferred = $.Deferred();
            var savedKey;

            if (bundles.contains("Encryption")) {
                var createEncryptionViewModel = new createEncryption();
                createEncryptionViewModel.creationEncryption.done(function (key, encryptionAlgorithm, encryptionBits, isEncryptedIndexes) {
                    savedKey = key;
                    securedSettings = {
                        'Raven/Encryption/Key': key,
                        'Raven/Encryption/Algorithm': _this.getEncryptionAlgorithmFullName(encryptionAlgorithm),
                        'Raven/Encryption/KeyBitsPreference': encryptionBits,
                        'Raven/Encryption/EncryptIndexes': isEncryptedIndexes
                    };
                    deferred.resolve(securedSettings);
                });
                app.showDialog(createEncryptionViewModel);
            } else {
                deferred.resolve({});
            }

            deferred.done(function () {
                new createDatabaseCommand(databaseName, settings, securedSettings).execute().done(function () {
                    _this.databases.unshift(new database(databaseName));
                    if (!jQuery.isEmptyObject(securedSettings)) {
                        var createEncryptionConfirmationViewModel = new createEncryptionConfirmation(savedKey);
                        app.showDialog(createEncryptionConfirmationViewModel);
                    }
                });
            });
        };

        databases.prototype.isEmptyStringOrWhitespace = function (str) {
            return !$.trim(str);
        };

        databases.prototype.getEncryptionAlgorithmFullName = function (encrytion) {
            var fullEncryptionName = null;
            switch (encrytion) {
                case "DES":
                    fullEncryptionName = "System.Security.Cryptography.DESCryptoServiceProvider, mscorlib";
                    break;
                case "R2C2":
                    fullEncryptionName = "System.Security.Cryptography.RC2CryptoServiceProvider, mscorlib";
                    break;
                case "Rijndael":
                    fullEncryptionName = "System.Security.Cryptography.RijndaelManaged, mscorlib";
                    break;
                default:
                    fullEncryptionName = "System.Security.Cryptography.TripleDESCryptoServiceProvider, mscorlib";
            }
            return fullEncryptionName;
        };

        databases.prototype.fetchStats = function (db) {
            new getDatabaseStatsCommand(db).execute().done(function (result) {
                return db.statistics(result);
            });
        };

        databases.prototype.selectDatabase = function (db) {
            this.databases().forEach(function (d) {
                return d.isSelected(d === db);
            });
            db.activate();
            this.selectedDatabase(db);
        };

        databases.prototype.goToDocuments = function (db) {
            router.navigate("#documents?database=" + encodeURIComponent(db.name));
        };

        databases.prototype.filterDatabases = function (filter) {
            var filterLower = filter.toLowerCase();
            this.databases().forEach(function (d) {
                var isMatch = !filter || (d.name.toLowerCase().indexOf(filterLower) >= 0);
                d.isVisible(isMatch);
            });

            var selectedDatabase = this.selectedDatabase();
            if (selectedDatabase && !selectedDatabase.isVisible()) {
                selectedDatabase.isSelected(false);
                this.selectedDatabase(null);
            }
        };

        databases.prototype.deleteSelectedDatabase = function () {
            var _this = this;
            var db = this.selectedDatabase();
            if (db) {
                require(["viewmodels/deleteDatabaseConfirm"], function (deleteDatabaseConfirm) {
                    var confirmDeleteVm = new deleteDatabaseConfirm(db, _this.systemDb);
                    confirmDeleteVm.deleteTask.done(function () {
                        return _this.onDatabaseDeleted(db);
                    });
                    app.showDialog(confirmDeleteVm);
                });
            }
        };

        databases.prototype.onDatabaseDeleted = function (db) {
            this.databases.remove(db);
            this.databases.valueHasMutated();
            if (this.databases.length === 0)
                this.selectDatabase(this.systemDb);
            else if (this.databases.contains(this.selectedDatabase()) === false) {
                this.selectDatabase(this.databases().first());
            }
        };
        return databases;
    })(viewModelBase);

    
    return databases;
});
