﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "viewmodels/viewModelBase", "models/transformer", "commands/saveTransformerCommand", "commands/getSingleTransformerCommand", "common/aceEditorBindingHandler", "viewmodels/deleteTransformerConfirm", "viewmodels/saveTransformerWithNewNameConfirm", "plugins/dialog", "common/appUrl", "plugins/router", "common/alertType", "common/alertArgs"], function(require, exports, viewModelBase, transformer, saveTransformerCommand, getSingleTransformerCommand, aceEditorBindingHandler, deleteTransformerConfirm, saveTransformerWithNewNameConfirm, dialog, appUrl, router, alertType, alertArgs) {
    var editTransformer = (function (_super) {
        __extends(editTransformer, _super);
        function editTransformer() {
            _super.call(this);
            this.editedTransformer = ko.observable();
            this.isEditingExistingTransformer = ko.observable(false);
            this.popoverOptions = ko.observable();
            this.editorCollection = ko.observableArray();
            aceEditorBindingHandler.install();
        }
        editTransformer.prototype.canActivate = function (transformerToEditName) {
            var _this = this;
            if (transformerToEditName) {
                var canActivateResult = $.Deferred();
                this.editExistingTransformer(transformerToEditName).done(function () {
                    return canActivateResult.resolve({ can: true });
                }).fail(function () {
                    ko.postbox.publish("Alert", new alertArgs(3 /* danger */, "Could not find " + transformerToEditName + " transformer", null));
                    canActivateResult.resolve({ redirect: appUrl.forTransformers(_this.activeDatabase()) });
                });

                return canActivateResult;
            } else {
                return $.Deferred().resolve({ can: true });
            }
        };

        editTransformer.prototype.activate = function (transformerToEditName) {
            _super.prototype.activate.call(this, transformerToEditName);

            if (transformerToEditName) {
                this.isEditingExistingTransformer(true);
            } else {
                this.editedTransformer(transformer.empty());
            }
        };

        editTransformer.prototype.attached = function () {
            var _this = this;
            this.addTransformerHelpPopover();
            this.createKeyboardShortcut("alt+c", function () {
                return _this.focusOnEditor();
            }, editTransformer.containerSelector);
            this.createKeyboardShortcut("alt+shift+del", function () {
                return _this.deleteTransformer();
            }, editTransformer.containerSelector);

            viewModelBase.dirtyFlag = new ko.DirtyFlag([this.editedTransformer().name, this.editedTransformer().transformResults]);
        };

        editTransformer.prototype.saveInObservable = function () {
        };

        editTransformer.prototype.addTransformerHelpPopover = function () {
            $("#transformerResultsLabel").popover({
                html: true,
                trigger: 'hover',
                content: 'The Transform function allows you to change the shape of individual result documents before the server returns them. It uses C# LINQ query syntax <br/> <br/> Example: <pre> <br/> <span class="code-keyword">from</span> order <span class="code-keyword">in</span> orders <br/> <span class="code-keyword">let</span> region = Database.Load(result.RegionId) <br/> <span class="code-keyword">select new</span> { <br/> result.Date, <br/> result.Amount, <br/> Region = region.Name, <br/> Manager = region.Manager <br/>}</pre>'
            });
        };

        editTransformer.prototype.focusOnEditor = function (elements, data) {
            if (typeof elements === "undefined") { elements = null; }
            if (typeof data === "undefined") { data = null; }
            var editorElement = $("#transformerAceEditor").length == 1 ? $("#transformerAceEditor")[0] : null;
            if (editorElement) {
                var docEditor = ko.utils.domData.get($("#transformerAceEditor")[0], "aceEditor");
                if (docEditor) {
                    docEditor.focus();
                }
            }
        };

        editTransformer.prototype.editExistingTransformer = function (unescapedTransformerName) {
            var _this = this;
            var indexName = decodeURIComponent(unescapedTransformerName);
            return this.fetchTransformerToEdit(indexName).done(function (trans) {
                return _this.editedTransformer(new transformer().initFromSave(trans));
            });
        };

        editTransformer.prototype.fetchTransformerToEdit = function (transformerName) {
            return new getSingleTransformerCommand(transformerName, this.activeDatabase()).execute();
        };

        editTransformer.prototype.saveTransformer = function () {
            var _this = this;
            if (this.isEditingExistingTransformer() && this.editedTransformer().wasNameChanged()) {
                var db = this.activeDatabase();
                var saveTransformerWithNewNameViewModel = new saveTransformerWithNewNameConfirm(this.editedTransformer(), db);
                saveTransformerWithNewNameViewModel.saveTask.done(function (trans) {
                    return _this.updateUrl(_this.editedTransformer().name());
                });
                dialog.show(saveTransformerWithNewNameViewModel);
            } else {
                new saveTransformerCommand(this.editedTransformer(), this.activeDatabase()).execute().done(function () {
                    if (!_this.isEditingExistingTransformer()) {
                        _this.isEditingExistingTransformer(true);
                        _this.updateUrl(_this.editedTransformer().name());
                    }
                });
            }

            viewModelBase.dirtyFlag().reset();
        };

        editTransformer.prototype.updateUrl = function (transformerName) {
            router.navigate(appUrl.forEditTransformer(transformerName, this.activeDatabase()));
        };

        editTransformer.prototype.deleteTransformer = function () {
            var transformer = this.editedTransformer();

            if (transformer) {
                var db = this.activeDatabase();
                var deleteViewmodel = new deleteTransformerConfirm([transformer.name()], db);
                deleteViewmodel.deleteTask.done(function () {
                    router.navigate(appUrl.forTransformers(db));
                });
                dialog.show(deleteViewmodel);
            }
        };
        editTransformer.containerSelector = "#editTransformerContainer";
        return editTransformer;
    })(viewModelBase);

    
    return editTransformer;
});
