﻿var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "viewmodels/dialogViewModelBase", "plugins/dialog", "commands/commandBase"], function(require, exports, dialogViewModelBase, dialog, commandBase) {
    var createFilesystem = (function (_super) {
        __extends(createFilesystem, _super);
        function createFilesystem(filesystems) {
            _super.call(this);
            this.creationTask = $.Deferred();
            this.creationTaskStarted = false;
            this.filesystemName = ko.observable('');
            this.filesystemPath = ko.observable('');
            this.filesystemLogs = ko.observable('');
            this.filesystemNameFocus = ko.observable(true);
            this.filesystems = ko.observableArray();
            this.newCommandBase = new commandBase();
            this.filesystems = filesystems;
        }
        createFilesystem.prototype.cancel = function () {
            dialog.close(this);
        };

        createFilesystem.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.filesystemNameFocus(true);
        };

        createFilesystem.prototype.deactivate = function () {
            if (!this.creationTaskStarted) {
                this.creationTask.reject();
            }
        };

        createFilesystem.prototype.nextOrCreate = function () {
            var filesystemName = this.filesystemName();

            if (this.isClientSideInputOK(filesystemName)) {
                this.creationTaskStarted = true;
                this.creationTask.resolve(filesystemName);
                dialog.close(this);
            }
        };

        createFilesystem.prototype.isClientSideInputOK = function (filesystemName) {
            var errorMessage = "";

            if (filesystemName == null) {
                errorMessage = "Please fill out the Filesystem Name field";
            } else if (this.isFilesystemNameExists(filesystemName, this.filesystems()) === true) {
                errorMessage = "Filesystem Name Already Exists!";
            } else if ((errorMessage = this.CheckInput(filesystemName)) != null) {
            }

            if (errorMessage != null) {
                this.newCommandBase.reportError(errorMessage);
                this.filesystemNameFocus(true);
                return false;
            }
            return true;
        };

        createFilesystem.prototype.CheckInput = function (name) {
            var rg1 = /^[^\\/:\*\?"<>\|]+$/;
            var rg2 = /^\./;
            var rg3 = /^(nul|prn|con|lpt[0-9]|com[0-9])(\.|$)/i;
            var maxLength = 260 - 30;

            var message = null;
            if (name.length > maxLength) {
                message = "The filesystem length can't exceed " + maxLength + " characters!";
            } else if (!rg1.test(name)) {
                message = "The filesystem name can't contain any of the following characters: \ / : * ?" + ' " ' + "< > |";
            } else if (rg2.test(name)) {
                message = "The filesystem name can't start with a dot!";
            } else if (rg3.test(name)) {
                message = "The name '" + name + "' is forbidden for use!";
            }
            return message;
        };

        createFilesystem.prototype.isFilesystemNameExists = function (filesystemName, filesystems) {
            for (var i = 0; i < filesystems.length; i++) {
                if (filesystemName == filesystems[i].name) {
                    return true;
                }
            }
            return false;
        };
        return createFilesystem;
    })(dialogViewModelBase);

    
    return createFilesystem;
});
