var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "viewmodels/viewModelBase", "commands/getPeriodicBackupSetupCommand", "commands/getDatabaseSettingsCommand", "models/periodicBackupSetup", "commands/savePeriodicBackupSetupCommand", "common/appUrl"], function(require, exports, viewModelBase, getPeriodicBackupSetupCommand, getDatabaseSettingsCommand, periodicBackupSetup, savePeriodicBackupSetupCommand, appUrl) {
    var periodicBackup = (function (_super) {
        __extends(periodicBackup, _super);
        function periodicBackup() {
            _super.apply(this, arguments);
            this.backupSetup = ko.observable().extend({ required: true });
            this.backupStatusDirtyFlag = new ko.DirtyFlag([]);
            this.backupConfigDirtyFlag = new ko.DirtyFlag([]);
        }
        periodicBackup.prototype.canActivate = function (args) {
            var _this = this;
            _super.prototype.canActivate.call(this, args);

            this.backupSetup(new periodicBackupSetup);
            var deferred = $.Deferred();
            var db = this.activeDatabase();
            if (db) {
                $.when(this.fetchPeriodicBackupSetup(db), this.fetchPeriodicBackupAccountsSettings(db)).done(function () {
                    return deferred.resolve({ can: true });
                }).fail(function () {
                    return deferred.resolve({ redirect: appUrl.forIndexes(_this.activeDatabase()) });
                });
            }
            return deferred;
        };

        periodicBackup.prototype.activate = function (args) {
            _super.prototype.activate.call(this, args);

            this.backupStatusDirtyFlag = new ko.DirtyFlag([this.backupSetup().disabled]);
            this.backupConfigDirtyFlag = new ko.DirtyFlag([this.backupSetup]);

            var self = this;
            this.isSaveEnabled = ko.computed(function () {
                return (self.backupConfigDirtyFlag().isDirty()) && (!self.backupSetup().disabled() || (self.backupSetup().disabled() && self.backupStatusDirtyFlag().isDirty()));
            });

            viewModelBase.dirtyFlag = new ko.DirtyFlag([this.isSaveEnabled]);
        };

        periodicBackup.prototype.fetchPeriodicBackupSetup = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getPeriodicBackupSetupCommand(db).execute().done(function (result) {
                return _this.backupSetup().fromDto(result);
            }).always(function () {
                return deferred.resolve({ can: true });
            });
            return deferred;
        };

        periodicBackup.prototype.fetchPeriodicBackupAccountsSettings = function (db) {
            var _this = this;
            var deferred = $.Deferred();
            new getDatabaseSettingsCommand(db).execute().done(function (document) {
                return _this.backupSetup().fromDatabaseSettingsDto(document.toDto());
            }).always(function () {
                return deferred.resolve({ can: true });
            });
            return deferred;
        };

        periodicBackup.prototype.activatePeriodicBackup = function () {
            var action = !this.backupSetup().disabled();
            this.backupSetup().disabled(action);
        };

        periodicBackup.prototype.saveChanges = function () {
            var _this = this;
            var db = this.activeDatabase();
            if (db) {
                var saveTask = new savePeriodicBackupSetupCommand(this.backupSetup(), db).execute();
                saveTask.done(function () {
                    _this.backupStatusDirtyFlag().reset();
                    _this.backupConfigDirtyFlag().reset();
                });
            }
        };
        return periodicBackup;
    })(viewModelBase);

    
    return periodicBackup;
});
