var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
define(["require", "exports", "models/document", "plugins/dialog", "viewmodels/dialogViewModelBase", 'models/customColumnParams', 'commands/saveDocumentCommand', 'commands/deleteDocumentCommand', 'commands/commandBase'], function(require, exports, document, dialog, dialogViewModelBase, customColumnParams, saveDocumentCommand, deleteDocumentCommand, commandBase) {
    var selectColumns = (function (_super) {
        __extends(selectColumns, _super);
        function selectColumns(customColumns, context, database) {
            _super.call(this);
            this.customColumns = customColumns;
            this.context = context;
            this.database = database;
            this.nextTask = $.Deferred();
            this.nextTaskStarted = false;
            this.newCommandBase = new commandBase();
        }
        selectColumns.prototype.attached = function () {
            _super.prototype.attached.call(this);
            this.form = $("#select-columns-form");
        };

        selectColumns.prototype.cancel = function () {
            dialog.close(this);
        };

        selectColumns.prototype.deactivate = function () {
            if (!this.nextTaskStarted) {
                this.nextTask.reject();
            }
        };

        selectColumns.prototype.onExit = function () {
            return this.nextTask.promise();
        };

        selectColumns.prototype.changeCurrentColumns = function () {
            this.nextTaskStarted = true;
            this.nextTask.resolve(this.customColumns);
            dialog.close(this);
        };

        selectColumns.prototype.insertNewRow = function () {
            this.customColumns.columns.push(customColumnParams.empty());
            this.alignBoxVertically();
        };

        selectColumns.prototype.deleteRow = function (row) {
            this.customColumns.columns.remove(row);
            this.alignBoxVertically();
        };

        selectColumns.prototype.moveUp = function (row) {
            var i = this.customColumns.columns.indexOf(row);
            if (i >= 1) {
                var array = this.customColumns.columns();
                this.customColumns.columns.splice(i - 1, 2, array[i], array[i - 1]);
            }
        };

        selectColumns.prototype.moveDown = function (row) {
            var i = this.customColumns.columns.indexOf(row);
            if (i >= 0 && i < this.customColumns.columns().length - 1) {
                var array = this.customColumns.columns();
                this.customColumns.columns.splice(i, 2, array[i + 1], array[i]);
            }
        };

        selectColumns.prototype.customScheme = function (val) {
            this.customColumns.customMode(val);
            this.alignBoxVertically();
        };

        selectColumns.prototype.alignBoxVertically = function () {
            var messageBoxHeight = parseInt($(".messageBox").css('height'), 10);
            var windowHeight = $(window).height();
            var messageBoxMarginTop = parseInt($(".messageBox").css('margin-top'), 10);
            var newTopPercent = Math.floor(((windowHeight - messageBoxHeight) / 2 - messageBoxMarginTop) / windowHeight * 100);
            var newTopPercentString = newTopPercent.toString() + '%';
            $(".modalHost").css('top', newTopPercentString);
        };

        selectColumns.prototype.saveAsDefault = function () {
            var _this = this;
            if (this.form[0].checkValidity() === true) {
                if (this.customColumns.customMode()) {
                    var configurationDocument = new document(this.customColumns.toDto());
                    new saveDocumentCommand(this.context, configurationDocument, this.database, false).execute().done(function () {
                        return _this.onConfigSaved();
                    }).fail(function () {
                        return _this.newCommandBase.reportError("Unable to save configuration!");
                    });
                } else {
                    new deleteDocumentCommand(this.context, this.database).execute().done(function () {
                        return _this.onConfigSaved();
                    }).fail(function () {
                        _this.newCommandBase.reportError("Unable to save configuration!");
                    });
                }
            } else {
                this.newCommandBase.reportWarning('Configuration contains errors. Not saving it.');
            }
        };

        selectColumns.prototype.onConfigSaved = function () {
            this.newCommandBase.reportSuccess('Configuration saved!');
        };
        return selectColumns;
    })(dialogViewModelBase);

    
    return selectColumns;
});
