define(["require", "exports", "common/appUrl", "plugins/router", "durandal/app", "models/uploadItem", "viewmodels/viewSystemDatabaseConfirm", "ace/ace"], function(require, exports, appUrl, router, app, uploadItem, viewSystemDatabaseConfirm, ace) {
    var viewModelBase = (function () {
        function viewModelBase() {
            this.activeDatabase = ko.observable().subscribeTo("ActivateDatabase", true);
            this.activeFilesystem = ko.observable().subscribeTo("ActivateFilesystem", true);
            this.keyboardShortcutDomContainers = [];
        }
        viewModelBase.prototype.canActivate = function (args) {
            var database = (appUrl.getDatabase() != null) ? appUrl.getDatabase() : appUrl.getSystemDatabase();

            if (database.isSystem) {
                if (viewModelBase.isConfirmedUsingSystemDatabase) {
                    return true;
                }

                return this.promptNavSystemDb();
            }

            viewModelBase.isConfirmedUsingSystemDatabase = false;

            return true;
        };

        viewModelBase.prototype.activate = function (args) {
            this.localStorageUploadQueueKey = "ravenFs-uploadQueue.";
            var db = appUrl.getDatabase();
            var currentDb = this.activeDatabase();
            if (!!db && db !== null && (!currentDb || currentDb.name !== db.name)) {
                ko.postbox.publish("ActivateDatabaseWithName", db.name);
            }

            var fs = appUrl.getFilesystem();
            var currentFilesystem = this.activeFilesystem();
            if (!currentFilesystem || currentFilesystem.name !== fs.name) {
                ko.postbox.publish("ActivateFilesystemWithName", fs.name);
            }

            this.modelPollingStart();

            window.onbeforeunload = function (e) {
                var isDirty = viewModelBase.dirtyFlag().isDirty();
                if (isDirty) {
                    var message = "You have unsaved data.";
                    e = e || window.event;

                    if (e) {
                        e.returnValue = message;
                    }

                    return message;
                }
                return null;
            };

            ko.postbox.publish("SetRawJSONUrl", "");
        };

        viewModelBase.prototype.attached = function () {
            webshims.polyfill();
        };

        viewModelBase.prototype.compositionComplete = function () {
            this.createResizableTextBoxes();
            viewModelBase.dirtyFlag().reset();
        };

        viewModelBase.prototype.canDeactivate = function (isClose) {
            var isDirty = viewModelBase.dirtyFlag().isDirty();
            if (isDirty) {
                return app.showMessage('You have unsaved data. Are you sure you want to close?', 'Unsaved Data', ['Yes', 'No']);
            }
            return true;
        };

        viewModelBase.prototype.deactivate = function () {
            var _this = this;
            this.activeDatabase.unsubscribeFrom("ActivateDatabase");
            this.activeFilesystem.unsubscribeFrom("ActivateFilesystem");
            this.keyboardShortcutDomContainers.forEach(function (el) {
                return _this.removeKeyboardShortcuts(el);
            });
            this.modelPollingStop();
        };

        viewModelBase.prototype.createResizableTextBoxes = function () {
            var self = this;
            $("pre").each(function () {
                self.createResizableTextBox(this);
            });

            $('#accordion').on('shown.bs.collapse', function () {
                var element = $(this).find('pre');
            });
        };

        viewModelBase.prototype.createResizableTextBox = function (element) {
            var editor = ace.edit(element);

            var minHeight = 100;
            if ($(element).height() < 150) {
                $(element).height(minHeight);
            }
            $(element).resizable({
                minHeight: minHeight,
                handles: "s, se",
                grid: [10000000000000000, 1],
                resize: function (event, ui) {
                    editor.resize();
                }
            });
            $(element).find('.ui-resizable-se').removeClass('ui-icon-gripsmall-diagonal-se');
            $(element).find('.ui-resizable-se').addClass('ui-icon-carat-1-s');
            $('.ui-resizable-se').css('cursor', 's-resize');
            window.onresize = function (event) {
                editor.resize();
            };
        };

        viewModelBase.prototype.createKeyboardShortcut = function (keys, handler, elementSelector) {
            jwerty.key(keys, function (e) {
                e.preventDefault();
                handler();
            }, this, elementSelector);

            if (!this.keyboardShortcutDomContainers.contains(elementSelector)) {
                this.keyboardShortcutDomContainers.push(elementSelector);
            }
        };

        viewModelBase.prototype.removeKeyboardShortcuts = function (elementSelector) {
            $(elementSelector).unbind('keydown.jwerty');
        };

        viewModelBase.prototype.navigate = function (url) {
            router.navigate(url);
        };

        viewModelBase.prototype.updateUrl = function (url) {
            var options = {
                replace: true,
                trigger: false
            };
            router.navigate(url, options);
        };

        viewModelBase.prototype.modelPolling = function () {
        };

        viewModelBase.prototype.forceModelPolling = function () {
            this.modelPolling();
        };

        viewModelBase.prototype.modelPollingStart = function () {
            var _this = this;
            this.modelPolling();
            this.activeDatabase.subscribe(function () {
                return _this.forceModelPolling();
            });
            this.activeFilesystem.subscribe(function () {
                return _this.forceModelPolling();
            });
        };

        viewModelBase.prototype.modelPollingStop = function () {
            clearInterval(this.modelPollingHandle);
        };

        viewModelBase.prototype.promptNavSystemDb = function () {
            if (!appUrl.warnWhenUsingSystemDatabase) {
                return true;
            }

            var canNavTask = $.Deferred();

            var systemDbConfirm = new viewSystemDatabaseConfirm();
            systemDbConfirm.viewTask.fail(function () {
                return canNavTask.resolve({ redirect: 'databases' });
            }).done(function () {
                viewModelBase.isConfirmedUsingSystemDatabase = true;
                canNavTask.resolve(true);
            });
            app.showDialog(systemDbConfirm);

            return canNavTask;
        };

        viewModelBase.prototype.stringifyUploadQueue = function (queue) {
            return ko.toJSON(queue);
        };

        viewModelBase.prototype.parseUploadQueue = function (queue, fs) {
            var stringArray = JSON.parse(queue);
            var uploadQueue = [];

            for (var i = 0; i < stringArray.length; i++) {
                uploadQueue.push(new uploadItem(stringArray[i]["id"], stringArray[i]["fileName"], stringArray[i]["status"], fs));
            }

            return uploadQueue;
        };

        viewModelBase.prototype.updateLocalStorage = function (x, fs) {
            window.localStorage.setItem(this.localStorageUploadQueueKey + fs.name, this.stringifyUploadQueue(x));
        };

        viewModelBase.prototype.updateQueueStatus = function (guid, status, queue) {
            var items = ko.utils.arrayFilter(queue, function (i) {
                return i.id() === guid;
            });
            if (items) {
                items[0].status(status);
            }
        };
        viewModelBase.dirtyFlag = new ko.DirtyFlag([]);
        return viewModelBase;
    })();

    
    return viewModelBase;
});
